import sys
import ROOT
import ObsDict
import TreeCreator
from array import array
from ROOT import gROOT, TMVA, TMath, TFile, TLorentzVector

#Main function of the classifier.
def main():
    #Check that user has indicated filenames
    if sys.argv[3] is "":
        raise ValueError("You must specify signal, background and data filenames!")
        
    #If user provided False as a fourth argument, TTree reprocessing will not be performed
    if len(sys.argv) == 5:
        if sys.argv[4] == "False": 
            pass
        elif sys.argv[4] == "True":
            TreeCreator.processTrees() #This function is launching TTree reprocessing, i.e. Px, Py, Pz, E -> mjj, pTjj, ...
        else:  
            raise ValueError("You must specify the correct reprocessing flag (True\False)")
    else:  
        raise ValueError("You must specify the reprocessing flag at the end of the command line (True/False)")
        
    print "=======> Launching BDT classifier..."
    
    #The definition of reprocessed input TTrees
    trainSignal = ROOT.TFile("Reproc_Signal.root", "read") #VBF
    trainBackgr = ROOT.TFile("Reproc_Background.root", "read") #ggF
    Data        = ROOT.TFile("Reproc_Data.root", "read")
    
    #Define the output file
    outputFile = ROOT.TFile("Output_BDT.root", "recreate")
    
    #Taking the TTrees from the input files
    treeSignal = trainSignal.Get("observablesTree")
    treeBackgr = trainBackgr.Get("observablesTree")
    treeData   =        Data.Get("observablesTree")
    
    #Define the TMVA factory object
    factory = TMVA.Factory("TMVAJob", outputFile)
    
    #Signal and background weights that will be used for training
    signalWeight = 1.0
    backgrWeight = 1.0
    
    #Adding signal, background and regression trees in the factory
    factory.AddSignalTree(treeSignal, signalWeight)
    factory.AddBackgroundTree(treeBackgr, backgrWeight)
    factory.AddRegressionTree(treeData, signalWeight)
    
    #Tell the factory how to use the training and testing events
    #factory.PrepareTrainingAndTestTree(mycuts, mycutb, "nTrain_Signal=0:nTrain_Background=0:SplitMode=Random:NormMode=NumEvents:!V")
    
    #Including all the variables that we indicated in ObsDict.py
    for var in ObsDict.obsList:
        factory.AddVariable(var, 'F')
        factory.AddTarget(var, 'F')
        
    #Apply additional cuts on the signal and background samples (can be different)
    mycuts = ""
    mycutb = ""
    
    #Booking the classifier method, it is possible to book several of them at once
    factory.BookMethod( TMVA.Types.kBDT, "BDTG",
    "!H:!V:NTrees=1000:BoostType=Grad:Shrinkage=0.30:UseBaggedBoost:GradBaggingFraction=0.6:SeparationType=GiniIndex:nCuts=20:PruneMethod=CostComplexity:PruneStrength=50:MaxDepth=2" )
    
    #Performing a training, testing and evaluation of all methods
    factory.TrainAllMethods()
    factory.TestAllMethods()
    factory.EvaluateAllMethods();
    
    outputFile.Close();
    
    print "=======> Wrote root file: " + outputFile.GetName()
    print "=======> TMVAClassificationCategory is done!"
    
    #Launching multi-class an regression GUI interfaces to check the plots
    if not gROOT.IsBatch(): 
        TMVA.TMVAMultiClassGui(outputFile.GetName())
        TMVA.TMVARegGui(outputFile.GetName())
    
    if not gROOT.IsBatch(): 
        TMVA.TMVARegGui(outputFile.GetName())
        TMVA.TMVAMultiClassGui(outputFile.GetName())
    
    input("=======> Opening GUI interface. Press any key to continue.")
    print "=======> Done!"
    
    return 0
 
if __name__ == "__main__":
    main()
