import sys
import ROOT
import ObsDict
from array import array

def processTrees( names = ["vPx", "vPy", "vPz", "vE"] ):
    print "=======> Initializing TTree reprocessor..."
    print "=======> Reading branches " + str(names) + "..."

    #Opening the files that will be reprocessed
    #These files contain only four branches, indicated in "names" list
    Signal      = ROOT.TFile(sys.argv[1], "read")
    Background  = ROOT.TFile(sys.argv[2], "read")
    Data        = ROOT.TFile(sys.argv[3], "read")
    fileList    = {Signal:"Signal", Background:"Background", Data:"Data"}

    print "=======> Creating input TTrees..."
    #Creating new files, that will be used for classification
    #These files will contain new branches, which correspond to the observables indicated in ObsDict.py
    fileN = 1
    for file in fileList: #Looping over the files (signal, background and data)
        print "=======> File " + sys.argv[fileN] + " is being reproduced..."
        outputFile = ROOT.TFile("Reproc_" + fileList[file] + ".root", "recreate")
        inputJetsTree = file.Get("jetsTree")
        inputBosonTree = file.Get("bosonTree")
        tree = ROOT.TTree("observablesTree", "observablesTree")
        varN = 0
        for var in ObsDict.obsList: #Looping over the observables we want to construct (see ObsDict.py)
            print "         Variable " + str(ObsDict.obsList[varN]) + " is calculating..."
            value = array('f', [0.]) #Creating a branch variable
            tree.Branch(var, value, var + "/F") #Creating the branch itself
            if inputJetsTree.GetEntries() != inputBosonTree.GetEntries(): 
                raise ValueError("The number of events is different in jets and boson TTrees! Check consistency of the samples!")
            for i in range(inputJetsTree.GetEntries()):
            #for events in inputJetsTree: #Looping over the events in input trees
                inputJetsTree.GetEntry(i)
                jet1 = ROOT.TLorentzVector() #Defining three TLorentzVector jets and 
                jet2 = ROOT.TLorentzVector() #filling them from the input trees
                jet1.SetPxPyPzE(inputJetsTree.vPx[0], inputJetsTree.vPy[0], inputJetsTree.vPz[0], inputJetsTree.vE[0])
                jet2.SetPxPyPzE(inputJetsTree.vPx[1], inputJetsTree.vPy[1], inputJetsTree.vPz[1], inputJetsTree.vE[1])
                if ( jet1.E() == - jet2.E() ) or ( jet1.Px() == jet2.Px() and jet1.Py() == jet2.Py() ) or ( jet1.Eta() == jet2.Eta() ): continue
                inputBosonTree.GetEntry(i)
                boson = ROOT.TLorentzVector()
                boson.SetPxPyPzE(inputBosonTree.vPx[0], inputBosonTree.vPy[0], inputBosonTree.vPz[0], inputBosonTree.vE[0])
                value[0] = ObsDict.observables(jet1, jet2, boson, var) #Taking the jets' four-vectors and calculating the observable
                tree.Fill() #Filling the event in new tree
            varN += 1
        outputFile.Write() #Writing the branch in tree
        outputFile.Close() #Close the output file
        print "=======> Done"
        fileN += 1
    print "=======> Input TTees have been created!"